@@ -6,7 +6,7 @@
 
 r_obj* finalize_atomic_scalar(struct collector* v_collector);
 r_obj* finalize_scalar(struct collector* v_collector);
-r_obj* finalize_vec(struct collector* v_collector);
+r_obj* finalize_vector(struct collector* v_collector);
 r_obj* finalize_variant(struct collector* v_collector);
 r_obj* finalize_row(struct collector* v_collector);
 r_obj* finalize_sub(struct collector* v_collector);
