@@ -11,6 +11,22 @@ bool is_data_frame(r_obj* x) {
   return r_inherits(x, "data.frame");
 }
 
+static inline
+r_obj* alloc_df(r_ssize n_rows, r_ssize n_cols, r_obj* col_names) {
+  r_obj* df = KEEP(r_alloc_list(n_cols));
+  r_attrib_poke_names(df, col_names);
+  r_init_tibble(df, n_rows);
+
+  FREE(1);
+  return(df);
+}
+
+static inline
+void r_poke_list_of(r_obj* x, r_obj* ptype) {
+  r_attrib_poke_class(x, classes_list_of);
+  r_attrib_poke(x, syms_ptype, ptype);
+}
+
 r_obj* r_list_get_by_name(r_obj* x, const char* nm);
 
 r_obj* apply_transform(r_obj* value, r_obj* fn);
@@ -27,6 +43,7 @@ r_obj* vec_flatten(r_obj* value, r_obj* ptype) {
 
 static inline
 r_obj* names2(r_obj* x) {
+  // simplified version of `rlang::ffi_names2()`
   r_obj* nms = r_names(x);
 
   if (nms == r_null) {
